@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2004  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005, 2007, 2009  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1999-2001  Internet Software Consortium.
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: t_db.c,v 1.29.206.3 2004/10/25 01:36:06 marka Exp $ */
+/* $Id: t_db.c,v 1.36.128.2 2009/01/22 23:46:35 tbox Exp $ */
 
 #include <config.h>
 
@@ -265,7 +265,7 @@ static void
 t1(void) {
 	int	result;
 
-	t_assert("dns_db_load", 1, T_REQUIRED, a1);
+	t_assert("dns_db_load", 1, T_REQUIRED, "%s", a1);
 	result = t_eval("dns_db_load_data", t_dns_db_load, 9);
 	t_result(result);
 }
@@ -403,8 +403,10 @@ test_dns_db_zc_x(const char *filename, dns_dbtype_t dbtype,
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = t_bustline(p, tokens);
 			if (cnt == 4) {
@@ -449,7 +451,7 @@ static void
 t2(void) {
 	int	result;
 
-	t_assert("dns_db_iscache", 2, T_REQUIRED, a2);
+	t_assert("dns_db_iscache", 2, T_REQUIRED, "%s", a2);
 	result = test_dns_db_zc_x("dns_db_iscache_1_data",
 				  dns_dbtype_cache, dns_db_iscache, ISC_TRUE);
 	t_result(result);
@@ -465,7 +467,7 @@ static void
 t3(void) {
 	int	result;
 
-	t_assert("dns_db_iscache", 3, T_REQUIRED, a3);
+	t_assert("dns_db_iscache", 3, T_REQUIRED, "%s", a3);
 	result = test_dns_db_zc_x("dns_db_iscache_2_data",
 				  dns_dbtype_zone, dns_db_iscache, ISC_FALSE);
 	t_result(result);
@@ -481,7 +483,7 @@ static void
 t4(void) {
 	int	result;
 
-	t_assert("dns_db_iszone", 4, T_REQUIRED, a4);
+	t_assert("dns_db_iszone", 4, T_REQUIRED, "%s", a4);
 	result = test_dns_db_zc_x("dns_db_iszone_1_data",
 				  dns_dbtype_zone, dns_db_iszone, ISC_TRUE);
 	t_result(result);
@@ -496,7 +498,7 @@ static void
 t5(void) {
 	int	result;
 
-	t_assert("dns_db_iszone", 5, T_REQUIRED, a5);
+	t_assert("dns_db_iszone", 5, T_REQUIRED, "%s", a5);
 	result = test_dns_db_zc_x("dns_db_iszone_2_data",
 				  dns_dbtype_cache, dns_db_iszone, ISC_FALSE);
 	t_result(result);
@@ -616,7 +618,7 @@ static void
 t6(void) {
 	int	result;
 
-	t_assert("dns_db_origin", 6, T_REQUIRED, a6);
+	t_assert("dns_db_origin", 6, T_REQUIRED, "%s", a6);
 	result = t_eval("dns_db_origin_data", t_dns_db_origin, 2);
 	t_result(result);
 }
@@ -730,7 +732,7 @@ static void
 t7(void) {
 	int	result;
 
-	t_assert("dns_db_class", 7, T_REQUIRED, a7);
+	t_assert("dns_db_class", 7, T_REQUIRED, "%s", a7);
 	result = t_eval("dns_db_class_data", t_dns_db_class, 2);
 	t_result(result);
 }
@@ -1003,7 +1005,7 @@ static void
 t8(void) {
 	int	result;
 
-	t_assert("dns_db_currentversion", 8, T_REQUIRED, a8);
+	t_assert("dns_db_currentversion", 8, T_REQUIRED, "%s", a8);
 	result = t_eval("dns_db_currentversion_data",
 			t_dns_db_currentversion, 7);
 	t_result(result);
@@ -1321,7 +1323,7 @@ static void
 t9(void) {
 	int	result;
 
-	t_assert("dns_db_newversion", 9, T_REQUIRED, a9);
+	t_assert("dns_db_newversion", 9, T_REQUIRED, "%s", a9);
 	result = t_eval("dns_db_newversion_data", t_dns_db_newversion, 7);
 	t_result(result);
 }
@@ -1732,7 +1734,7 @@ static void
 t10(void) {
 	int	result;
 
-	t_assert("dns_db_closeversion", 10, T_REQUIRED, a10);
+	t_assert("dns_db_closeversion", 10, T_REQUIRED, "%s", a10);
 	result = t_eval("dns_db_closeversion_1_data",
 			t_dns_db_closeversion_1, 9);
 	t_result(result);
@@ -2207,7 +2209,7 @@ static void
 t11(void) {
 	int	result;
 
-	t_assert("dns_db_closeversion", 11, T_REQUIRED, a11);
+	t_assert("dns_db_closeversion", 11, T_REQUIRED, "%s", a11);
 	result = t_eval("dns_db_closeversion_2_data",
 			t_dns_db_closeversion_2, 9);
 	t_result(result);
@@ -2412,7 +2414,7 @@ static void
 t12(void) {
 	int	result;
 
-	t_assert("dns_db_expirenode", 12, T_REQUIRED, a12);
+	t_assert("dns_db_expirenode", 12, T_REQUIRED, "%s", a12);
 	result = t_eval("dns_db_expirenode_data", t_dns_db_expirenode, 8);
 	t_result(result);
 }
@@ -2578,7 +2580,7 @@ static void
 t13(void) {
 	int	result;
 
-	t_assert("dns_db_findnode", 13, T_REQUIRED, a13);
+	t_assert("dns_db_findnode", 13, T_REQUIRED, "%s", a13);
 	result = t_eval("dns_db_findnode_1_data", t_dns_db_findnode_1, 8);
 	t_result(result);
 }
@@ -2776,7 +2778,7 @@ static void
 t14(void) {
 	int	result;
 
-	t_assert("dns_db_findnode", 14, T_REQUIRED, a14);
+	t_assert("dns_db_findnode", 14, T_REQUIRED, "%s", a14);
 	result = t_eval("dns_db_findnode_2_data", t_dns_db_findnode_2, 6);
 	t_result(result);
 }
@@ -2975,7 +2977,7 @@ static void
 t15(void) {
 	int	result;
 
-	t_assert("dns_db_find", 15, T_REQUIRED, a15);
+	t_assert("dns_db_find", 15, T_REQUIRED, "%s", a15);
 	result = t_eval("dns_db_find_1_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -2991,7 +2993,7 @@ static void
 t16(void) {
 	int	result;
 
-	t_assert("dns_db_find", 16, T_REQUIRED, a16);
+	t_assert("dns_db_find", 16, T_REQUIRED, "%s", a16);
 	result = t_eval("dns_db_find_2_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -3004,7 +3006,7 @@ static void
 t17(void) {
 	int	result;
 
-	t_assert("dns_db_find", 17, T_REQUIRED, a17);
+	t_assert("dns_db_find", 17, T_REQUIRED, "%s", a17);
 	result = t_eval("dns_db_find_3_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -3017,7 +3019,7 @@ static void
 t18(void) {
 	int	result;
 
-	t_assert("dns_db_find", 18, T_REQUIRED, a18);
+	t_assert("dns_db_find", 18, T_REQUIRED, "%s", a18);
 	result = t_eval("dns_db_find_4_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -3030,7 +3032,7 @@ static void
 t19(void) {
 	int	result;
 
-	t_assert("dns_db_find", 19, T_REQUIRED, a19);
+	t_assert("dns_db_find", 19, T_REQUIRED, "%s", a19);
 	result = t_eval("dns_db_find_5_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -3043,7 +3045,7 @@ static void
 t20(void) {
 	int	result;
 
-	t_assert("dns_db_find", 20, T_REQUIRED, a20);
+	t_assert("dns_db_find", 20, T_REQUIRED, "%s", a20);
 	result = t_eval("dns_db_find_6_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -3056,7 +3058,7 @@ static void
 t21(void) {
 	int	result;
 
-	t_assert("dns_db_find", 21, T_REQUIRED, a21);
+	t_assert("dns_db_find", 21, T_REQUIRED, "%s", a21);
 	result = t_eval("dns_db_find_7_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -3069,7 +3071,7 @@ static void
 t22(void) {
 	int	result;
 
-	t_assert("dns_db_find", 22, T_REQUIRED, a22);
+	t_assert("dns_db_find", 22, T_REQUIRED, "%s", a22);
 	result = t_eval("dns_db_find_8_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -3083,7 +3085,7 @@ static void
 t23(void) {
 	int	result;
 
-	t_assert("dns_db_find", 23, T_REQUIRED, a23);
+	t_assert("dns_db_find", 23, T_REQUIRED, "%s", a23);
 	result = t_eval("dns_db_find_9_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -3096,7 +3098,7 @@ static void
 t24(void) {
 	int	result;
 
-	t_assert("dns_db_find", 24, T_REQUIRED, a24);
+	t_assert("dns_db_find", 24, T_REQUIRED, "%s", a24);
 	result = t_eval("dns_db_find_10_data", t_dns_db_find_x, 10);
 	t_result(result);
 }
@@ -3109,7 +3111,7 @@ static void
 t25(void) {
 	int	result;
 
-	t_assert("dns_db_load", 25, T_REQUIRED, a25);
+	t_assert("dns_db_load", 25, T_REQUIRED, "%s", a25);
 	result = t_eval("dns_db_load_soa_not_top", t_dns_db_load, 9);
 	t_result(result);
 }
