@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2004  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2009  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1998-2003  Internet Software Consortium.
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: t_names.c,v 1.32.2.2.8.3 2004/03/08 21:06:23 marka Exp $ */
+/* $Id: t_names.c,v 1.43.128.5 2009/01/22 23:46:35 tbox Exp $ */
 
 #include <config.h>
 
@@ -38,7 +38,10 @@
 
 static char	*Tokens[MAXTOKS + 1];
 
-/*
+
+#ifdef	NEED_PBUF
+
+/*%
  * get a hex formatted dns message from a data
  * file into an isc_buffer_t
  * caller supplies data storage and the isc_buffer
@@ -46,8 +49,6 @@ static char	*Tokens[MAXTOKS + 1];
  * and return the data length
  */
 
-#ifdef	NEED_PBUF
-
 static char *
 ctoh(unsigned char c) {
 	int		val;
@@ -85,7 +86,7 @@ pbuf(isc_buffer_t *pbuf) {
 
 #endif	/* NEED_PBUF */
 
-/*
+/*%
  * Compare data at buf with data in hex representation at exp_data,
  * of length exp_data_len, for equality.
  * Return 0 if equal, else non-zero.
@@ -167,7 +168,7 @@ chkdata(unsigned char *buf, size_t buflen, char *exp_data,
 	return (result);
 }
 
-/*
+/*%
  * Get a hex formatted dns message from a data file into an isc_buffer_t.
  * Caller supplies data storage and the isc_buffer.  We read the file, convert,
  * setup the buffer and return the data length.
@@ -208,6 +209,7 @@ getmsg(char *datafile_name, unsigned char *buf, int buflen, isc_buffer_t *pbuf)
 		else if (('A' <= c) && (c <= 'Z'))
 			val = c - 'A'+ 10;
 		else {
+			(void)fclose(fp);
 			t_info("Bad format in datafile\n");
 			return (0);
 		}
@@ -221,6 +223,7 @@ getmsg(char *datafile_name, unsigned char *buf, int buflen, isc_buffer_t *pbuf)
 				/*
 				 * Buffer too small.
 				 */
+				(void)fclose(fp);
 				t_info("Buffer overflow error\n");
 				return (0);
 			}
@@ -256,14 +259,15 @@ bustline(char *line, char **toks) {
 	return (cnt);
 }
 
-/*
+
+#ifdef	NEED_HNAME_TO_TNAME
+
+/*%
  * convert a name from hex representation to text form
  * format of hex notation is:
  * %xXXXXXXXX
  */
 
-#ifdef	NEED_HNAME_TO_TNAME
-
 static int
 hname_to_tname(char *src, char *target, size_t len) {
 	int		i;
@@ -317,7 +321,7 @@ hname_to_tname(char *src, char *target, size_t len) {
 
 #endif	/* NEED_HNAME_TO_TNAME */
 
-/*
+/*%
  * initialize a dns_name_t from a text name, hiding all
  * buffer and other object initialization from the caller
  *
@@ -362,7 +366,7 @@ t_dns_name_init(void) {
 	unsigned char	offsets[1];
 
 	rval = 0;
-	t_assert("dns_name_init", 1, T_REQUIRED, a3);
+	t_assert("dns_name_init", 1, T_REQUIRED, "%s", a3);
 
 	dns_name_init(&name, offsets);
 	/* magic is hidden in name.c ...
@@ -412,7 +416,7 @@ t_dns_name_invalidate(void) {
 	dns_name_t	name;
 	unsigned char	offsets[1];
 
-	t_assert("dns_name_invalidate", 1, T_REQUIRED, a4);
+	t_assert("dns_name_invalidate", 1, T_REQUIRED, "%s", a4);
 
 	rval = 0;
 	dns_name_init(&name, offsets);
@@ -466,7 +470,7 @@ t_dns_name_setbuffer(void) {
 	dns_name_t	name;
 	isc_buffer_t	buffer;
 
-	t_assert("dns_name_setbuffer", 1, T_REQUIRED, a5);
+	t_assert("dns_name_setbuffer", 1, T_REQUIRED, "%s", a5);
 
 	isc_buffer_init(&buffer, junk, BUFLEN);
 	dns_name_init(&name, NULL);
@@ -490,7 +494,7 @@ t_dns_name_hasbuffer(void) {
 	dns_name_t	name;
 	isc_buffer_t	buffer;
 
-	t_assert("dns_name_hasbuffer", 1, T_REQUIRED, a6);
+	t_assert("dns_name_hasbuffer", 1, T_REQUIRED, "%s", a6);
 
 	rval = 0;
 	isc_buffer_init(&buffer, junk, BUFLEN);
@@ -553,7 +557,7 @@ t_dns_name_isabsolute(void) {
 	char	*p;
 	FILE	*fp;
 
-	t_assert("dns_name_isabsolute", 1, T_REQUIRED, a7);
+	t_assert("dns_name_isabsolute", 1, T_REQUIRED, "%s", a7);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_isabsolute_data", "r");
@@ -566,8 +570,10 @@ t_dns_name_isabsolute(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 2) {
@@ -575,7 +581,7 @@ t_dns_name_isabsolute(void) {
 				 * label, bitpos, expected value.
 				 */
 				result = test_dns_name_isabsolute(Tokens[0],
-							        atoi(Tokens[1])
+								atoi(Tokens[1])
 								  == 0 ?
 								  ISC_FALSE :
 								  ISC_TRUE);
@@ -598,7 +604,7 @@ static const char *a8 =	"dns_name_hash(name, case_sensitive) returns "
 		"a hash of 'name' which is case_sensitive if case_sensitive "
 		"is true";
 
-/*
+/*%
  * a9 merged with a8.
  */
 
@@ -664,7 +670,7 @@ t_dns_name_hash(void) {
 	char	*p;
 	FILE	*fp;
 
-	t_assert("dns_name_hash", 1, T_REQUIRED, a8);
+	t_assert("dns_name_hash", 1, T_REQUIRED, "%s", a8);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_hash_data", "r");
@@ -677,8 +683,10 @@ t_dns_name_hash(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 4) {
@@ -718,7 +726,7 @@ static const char *a10 =
 		"and sets nbitsp to the number of bits name1 and name2 "
 		"have in common";
 
-/*
+/*%
  * a11 thru a22 merged into a10.
  */
 static const char *
@@ -818,7 +826,7 @@ t_dns_name_fullcompare(void) {
 	FILE		*fp;
 	dns_namereln_t	reln;
 
-	t_assert("dns_name_fullcompare", 1, T_REQUIRED, a10);
+	t_assert("dns_name_fullcompare", 1, T_REQUIRED, "%s", a10);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_fullcompare_data", "r");
@@ -831,8 +839,10 @@ t_dns_name_fullcompare(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 6) {
@@ -880,7 +890,7 @@ static const char *a23 =
 		"the relative ordering under the DNSSEC ordering relationship "
 		"of name1 and name2";
 
-/*
+/*%
  * a24 thru a29 merged into a23.
  */
 
@@ -936,7 +946,7 @@ t_dns_name_compare(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_compare", 1, T_REQUIRED, a23);
+	t_assert("dns_name_compare", 1, T_REQUIRED, "%s", a23);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_compare_data", "r");
@@ -949,8 +959,10 @@ t_dns_name_compare(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 3) {
@@ -981,7 +993,7 @@ static const char *a30 =
 		"about the relative ordering of name1 and name2 as if they "
 		"are part of rdata in DNSSEC canonical form";
 
-/*
+/*%
  * a31, a32 merged into a30.
  */
 
@@ -1036,7 +1048,7 @@ t_dns_name_rdatacompare(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_rdatacompare", 1, T_REQUIRED, a30);
+	t_assert("dns_name_rdatacompare", 1, T_REQUIRED, "%s", a30);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_rdatacompare_data", "r");
@@ -1049,8 +1061,10 @@ t_dns_name_rdatacompare(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 3) {
@@ -1082,7 +1096,7 @@ static const char *a33 =
 		"dns_name_issubdomain(name1, name2) returns true, "
 		"otherwise it returns false.";
 
-/*
+/*%
  * a34 merged into a33.
  */
 
@@ -1132,7 +1146,7 @@ t_dns_name_issubdomain(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_issubdomain", 1, T_REQUIRED, a33);
+	t_assert("dns_name_issubdomain", 1, T_REQUIRED, "%s", a33);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_issubdomain_data", "r");
@@ -1145,8 +1159,10 @@ t_dns_name_issubdomain(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 3) {
@@ -1213,7 +1229,7 @@ t_dns_name_countlabels(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_countlabels", 1, T_REQUIRED, a35);
+	t_assert("dns_name_countlabels", 1, T_REQUIRED, "%s", a35);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_countlabels_data", "r");
@@ -1226,8 +1242,10 @@ t_dns_name_countlabels(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 2) {
@@ -1255,7 +1273,7 @@ static const char *a36 =
 		"dns_name_getlabel(name, n, labelp) initializes labelp "
 		"to point to the nth label in name";
 
-/*
+/*%
  * The strategy here is two take two dns names with a shared label in
  * different positions, get the two labels and compare them for equality.
  * If they don't match, dns_name_getlabel failed.
@@ -1323,7 +1341,7 @@ t_dns_name_getlabel(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_getlabel", 1, T_REQUIRED, a36);
+	t_assert("dns_name_getlabel", 1, T_REQUIRED, "%s", a36);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_getlabel_data", "r");
@@ -1336,8 +1354,10 @@ t_dns_name_getlabel(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 4) {
@@ -1346,7 +1366,7 @@ t_dns_name_getlabel(void) {
 				 */
 				result = test_dns_name_getlabel(Tokens[0],
 							      atoi(Tokens[1]),
-							           Tokens[2],
+								   Tokens[2],
 							      atoi(Tokens[3]));
 			} else {
 				t_info("bad format at line %d\n", line);
@@ -1368,7 +1388,7 @@ static const char *a37 =
 		"initializes target to point to the n label sequence of "
 		"labels in source starting with first";
 
-/*
+/*%
  * We adopt a similiar strategy to that used by the dns_name_getlabel test.
  */
 
@@ -1451,7 +1471,7 @@ t_dns_name_getlabelsequence(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_getlabelsequence", 1, T_REQUIRED, a37);
+	t_assert("dns_name_getlabelsequence", 1, T_REQUIRED, "%s", a37);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_getlabelsequence_data", "r");
@@ -1464,8 +1484,10 @@ t_dns_name_getlabelsequence(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 5) {
@@ -1539,7 +1561,7 @@ t_dns_name_fromregion(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_fromregion", 1, T_REQUIRED, a38);
+	t_assert("dns_name_fromregion", 1, T_REQUIRED, "%s", a38);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_fromregion_data", "r");
@@ -1552,8 +1574,10 @@ t_dns_name_fromregion(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 1) {
@@ -1587,7 +1611,7 @@ t_dns_name_toregion(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_toregion", 1, T_REQUIRED, a39);
+	t_assert("dns_name_toregion", 1, T_REQUIRED, "%s", a39);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_toregion_data", "r");
@@ -1600,8 +1624,10 @@ t_dns_name_toregion(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 1) {
@@ -1722,7 +1748,7 @@ t_dns_name_fromtext(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_fromtext", 1, T_REQUIRED, a40);
+	t_assert("dns_name_fromtext", 1, T_REQUIRED, "%s", a40);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_fromtext_data", "r");
@@ -1735,8 +1761,10 @@ t_dns_name_fromtext(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 4) {
@@ -1856,7 +1884,7 @@ t_dns_name_totext(void) {
 	char		*p;
 	FILE		*fp;
 
-	t_assert("dns_name_totext", 1, T_REQUIRED, a41);
+	t_assert("dns_name_totext", 1, T_REQUIRED, "%s", a41);
 
 	result = T_UNRESOLVED;
 	fp = fopen("dns_name_totext_data", "r");
@@ -1869,8 +1897,10 @@ t_dns_name_totext(void) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 2) {
@@ -1933,10 +1963,6 @@ static const char *a48 =
 		"returns ISC_R_UNEXPECTEDEND";
 
 static const char *a49 =
-		"when there are too many compression pointers, "
-		"dns_name_fromwire() returns DNS_R_TOOMANYHOPS";
-
-static const char *a50 =
 		"when there is not enough space in target, "
 		"dns_name_fromwire(name, source, dcts, downcase, target) "
 		"returns ISC_R_NOSPACE";
@@ -2028,8 +2054,10 @@ t_dns_name_fromwire_x(const char *testfile, size_t buflen) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 6) {
@@ -2090,7 +2118,7 @@ t_dns_name_fromwire_x(const char *testfile, size_t buflen) {
 
 static void
 t_dns_name_fromwire(void) {
-	t_assert("dns_name_fromwire", 1, T_REQUIRED, a42);
+	t_assert("dns_name_fromwire", 1, T_REQUIRED, "%s", a42);
 	t_dns_name_fromwire_x("dns_name_fromwire_1_data", BUFLEN);
 
 #if 0
@@ -2098,30 +2126,27 @@ t_dns_name_fromwire(void) {
 	 * XXXRTH these tests appear to be broken, so I have
 	 * disabled them.
 	 */
-	t_assert("dns_name_fromwire", 2, T_REQUIRED, a43);
+	t_assert("dns_name_fromwire", 2, T_REQUIRED, "%s", a43);
 	t_dns_name_fromwire_x("dns_name_fromwire_2_data", BUFLEN);
 
-	t_assert("dns_name_fromwire", 3, T_REQUIRED, a44);
+	t_assert("dns_name_fromwire", 3, T_REQUIRED, "%s", a44);
 	t_dns_name_fromwire_x("dns_name_fromwire_3_data", BUFLEN);
 #endif
 
-	t_assert("dns_name_fromwire", 4, T_REQUIRED, a45);
+	t_assert("dns_name_fromwire", 4, T_REQUIRED, "%s", a45);
 	t_dns_name_fromwire_x("dns_name_fromwire_4_data", BUFLEN);
 
-	t_assert("dns_name_fromwire", 5, T_REQUIRED, a46);
+	t_assert("dns_name_fromwire", 5, T_REQUIRED, "%s", a46);
 	t_dns_name_fromwire_x("dns_name_fromwire_5_data", BUFLEN);
 
-	t_assert("dns_name_fromwire", 6, T_REQUIRED, a47);
+	t_assert("dns_name_fromwire", 6, T_REQUIRED, "%s", a47);
 	t_dns_name_fromwire_x("dns_name_fromwire_6_data", BUFLEN);
 
-	t_assert("dns_name_fromwire", 7, T_REQUIRED, a48);
+	t_assert("dns_name_fromwire", 7, T_REQUIRED, "%s", a48);
 	t_dns_name_fromwire_x("dns_name_fromwire_7_data", BUFLEN);
 
-	t_assert("dns_name_fromwire", 8, T_REQUIRED, a49);
-	t_dns_name_fromwire_x("dns_name_fromwire_8_data", BUFLEN);
-
-	t_assert("dns_name_fromwire", 9, T_REQUIRED, a50);
-	t_dns_name_fromwire_x("dns_name_fromwire_9_data", 2);
+	t_assert("dns_name_fromwire", 9, T_REQUIRED, "%s", a49);
+	t_dns_name_fromwire_x("dns_name_fromwire_8_data", 2);
 }
 
 
@@ -2221,8 +2246,10 @@ t_dns_name_towire_x(const char *testfile, size_t buflen) {
 			/*
 			 * Skip comment lines.
 			 */
-			if ((isspace((unsigned char)*p)) || (*p == '#'))
+			if ((isspace((unsigned char)*p)) || (*p == '#')) {
+				(void)free(p);
 				continue;
+			}
 
 			cnt = bustline(p, Tokens);
 			if (cnt == 5) {
@@ -2257,13 +2284,13 @@ t_dns_name_towire_x(const char *testfile, size_t buflen) {
 
 static void
 t_dns_name_towire_1(void) {
-	t_assert("dns_name_towire", 1, T_REQUIRED, a51);
+	t_assert("dns_name_towire", 1, T_REQUIRED, "%s", a51);
 	t_dns_name_towire_x("dns_name_towire_1_data", BUFLEN);
 }
 
 static void
 t_dns_name_towire_2(void) {
-	t_assert("dns_name_towire", 2, T_REQUIRED, a52);
+	t_assert("dns_name_towire", 2, T_REQUIRED, "%s", a52);
 	t_dns_name_towire_x("dns_name_towire_2_data", 2);
 }
 
@@ -2284,7 +2311,7 @@ static const char *a53 =
 
 static void
 t_dns_name_concatenate(void) {
-	t_assert("dns_name_concatenate", 1, T_REQUIRED, a53);
+	t_assert("dns_name_concatenate", 1, T_REQUIRED, "%s", a53);
 	t_result(T_UNTESTED);
 }
 #endif
